@@ -4,10 +4,12 @@
 // Use of this source code is governed by a BSD-style
 // license that can be found in the LICENSE file.
 
+// +build !rtems
+
 package syscall
 
 func Sleep(nsec int64) (err error) {
-	tv := NsecToTimeval(nsec);
-	_, err = Select(0, nil, nil, nil, &tv);
-	return err;
+	tv := NsecToTimeval(nsec)
+	_, err = Select(0, nil, nil, nil, &tv)
+	return err
 }
