@@ -1,12 +1,12 @@
-// { dg-options "-std=gnu++0x" }
-// { dg-do compile }
+// { dg-options "-Wno-deprecated" }
+// { dg-do compile { target c++11 } }
 
-// Copyright (C) 2005, 2006, 2007 Free Software Foundation
+// Copyright (C) 2005-2018 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2, or (at your option)
+// Free Software Foundation; either version 3, or (at your option)
 // any later version.
 
 // This library is distributed in the hope that it will be useful,
@@ -15,9 +15,8 @@
 // GNU General Public License for more details.
 
 // You should have received a copy of the GNU General Public License along
-// with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-// USA.
+// with this library; see the file COPYING3.  If not see
+// <http://www.gnu.org/licenses/>.
 
 // 20.6.6.2 Template class shared_ptr [util.smartptr.shared]
 
@@ -33,11 +32,9 @@ struct B { };
 int
 test01()
 {
-  bool test __attribute__((unused)) = true;
-
   std::shared_ptr<A> a;
   std::auto_ptr<B> b;
-  a = b;                      // { dg-error "here" }
+  a = std::move(b);                      // { dg-error "no match" }
 
   return 0;
 }
@@ -48,4 +45,5 @@ main()
   test01();
   return 0;
 }
-// { dg-excess-errors "In constructor" }
+
+// { dg-prune-output "cannot convert" }
