@@ -1,4 +1,4 @@
-// Copyright (C) 2003-2017 Free Software Foundation, Inc.
+// Copyright (C) 2003-2018 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -49,8 +49,7 @@ protected:
 };
 
 // libstdc++/12790
-// basic_filebuf::seekpos() should call codecvt::unshift(),
-// but only if writing
+// basic_filebuf::seekpos() should call codecvt::unshift()
 void test01()
 {
   using namespace std;
@@ -65,10 +64,11 @@ void test01()
 
   fb.open(name, ios_base::out);
   streampos pos = fb.pubseekoff(0, ios_base::beg);
+  fb.sputc(L'a');
 
   VERIFY( !cvt->unshift_called );
   fb.pubseekpos(pos);
-  VERIFY( !cvt->unshift_called );
+  VERIFY( cvt->unshift_called );
 }
 
 int main()
